/*
BaudotKeyboardRTTYXmtr_V6
March 15, 2019
Ralph Irons N4RLI
n4rli@arrl.org

This Arduino Leonardo project uses push buttons to simulate a keyboard of the type patented by Emile Baudot in 1867 as part of a multiplexed telegraph system.
An Etherkit or Adafruit Si5351A digital clock generator board is used to produce RTTY, which drives an rf amplifier chain. (I am using a preamp design and a 
5 W amp kit from QRP Labs.)

Keyboard:
(built on two breadboards, one for each hand; or on an Arduino Shield PCB designed to replace them)

  Materials:
  1 PC
  1 Arduino Leonardo with USB cable
  1 Etherkit or Adafruit Si5351A breakout board
  2 breadboards (or a Baudot Keyboard Adruino Shield -- get Eagle schematic and board layout from www.mathmage.org/Baudot)
  16 male/male and 4 female/male Dupont jumper wires (the Baudot Keyboard Adruino Shield reuires 4 female/female and 2 male/female jumpers)
  6 breadboard pushbuttons (or 6 Cherry MX compatible keyboard switches and key caps, for use with the Baudot Keyboard Adruino Shield)
  1 red LED
  1 green LED
  2 220 ohm resistors
                                                                                                                                                                                                                                                                                                                                                          SMA Connector      _____                       
    GND                   GND               Pin 10              Pin 9         GND                  GND                  GND                   GND                       +5V Arduino            SCL Arduino                         +Vin Arduino                                          +13.8V Pwr Supply                                                 to Antenna         \|/
     |   Pin 12            |    Pin 11       |                  |              |     Pin 4         |    Pin 5            |    Pin 6            |    Pin 7                  |   GND Arduino      |    SDA Arduino                       |                   GND                               |                   GND                                            |  |-----------|
     |     |               |     |           \                  \              |     |             |     |               |     |               |     |                     |      |             |      |                               |                    |                                |                    |                                             |  |
     |     |               |     |           /220 ohm           /220 ohm       |     |             |     |               |     |               |     |                   |------------------------------------|                  |-----------------------------------|                  |-----------------------------------|             |----------------------------|                            
     |     |               |     |           \                  \              |     |             |     |               |     |               |     |                   |+5V    GND            SCL    SDA    |                  |   +5V                   GND       |                  |   +13.8V                GND       |             |                   Output   |                            
     |     |               |     |           | Green LED        | Red LED      |     |pb           |     | pb            |     | pb            |     | pb                |              CLK0 Output           |                  |      Input                 Output |                  |         Input             Output  |             |     Input                  |
     o     o               o     o         \   /   /          \   /   /        O     O             o     o               o     o               o     o                   |------------------------------------|                  |-----------------------------------|                  |-----------------------------------|             |----------------------------|
     _______               _______          \ /  __\           \ /  __\        _______             _______               _______               _______                                     |   |                                       |   |                   |   |                           |   |                |   |                      |   |
        |                     |             _v_    /           _v_    /           |                   |                     |                     |                                   SMA Connector                                SMA Connector            SMA Connector                   SMA Connector         SMA Connector               SMA Connector
       ---                   ---             |                  |                ---                 ---                   ---                   ---                                    to RF Preamp                               from Si5351              to RF Final                      from RF Preamp     to Low Pass Filter             to Final Amp
      key 5                 key 4           GND                GND               sw1                key 1                 key 2                 key 3                          
   ________________________________________________________________________________________      ______________________________________________________________  _______________________________________________________________________________________________________________________________________________________________________________________________________________________
                                  Breadboard for left hand                                                        Breadboard for right hand                                            Si5351A                                             RF Preamp                                                 Final Amp                                 Low Pass Filter
   

 * (Modified) Baudot Code (In suggested order for learning in short groups)
 * 
 * (Letter, Figure) key5key4 key1key2key3    X = depressed, O = not depressed
 * 
 * (A,1) OO XOO
 * (E,2) OO OXO
 * (I, ) OO OXX
 * (O,5) OO XXX
 * (U,4) OO XOX
 * (Y,3) OO OOX
 * 
 * (B,8) OX OOX
 * (C,9) OX XOX
 * (D,0) OX XXX
 * (F, ) OX OXX  //Original ita1 FIG character no longer in use in ita2
 * (G,7) OX OXO
 * (H, ) OX XXO  //Original ita1 FIG character no longer in use in ita2
 * 
 * (J,6) OX XOO
 * [K,(] XX X00
 * (L, ) XX XX0  //Original ita1 FIG character no longer in use in ita2
 * [M,)] XX OXO
 * (N, ) XX OXX  //Original ita1 FIG character no longer in use in ita2
 * (P, ) XX XXX  //Original ita1 FIG character no longer in use in ita2
 * 
 * (Q,/) XX XOX
 * (R,-) XX OOX
 * (S,;) XO OOX
 * (T,!) XO XOX
 * (V,') XO XXX
 * (W,?) XO OXX
 * 
 * (X,,) XO OXO
 * (Z,:) XO XXO
 * (BS,.) XO XOO //Original ita1 LTR char replaced with Backspace. Only works in buffer or wordprocesser--rtty has no backspace!
 * 
 * (FIG,FIG)     OX OOO
 * (LTR,LTR)     XO OOO
 * (CR LF,CR LF) XX OOO  //Original ita1 Baudot code has no carriage return or line feed; receiver printed onto continuous tape
 * (SPACE,SPACE) OO XXO  //Original ita1 Baudot code used LTR and FIG to send spaces; not suitable for ham callsigns

By default, the keyboard operates at approximately 2.1 baud, or about 5 WPM. Speed can be cycled through 5 WPM, 10 WPM, 15 WPM 
and 20 WPM (see Keyboard Macros below).

Keyboard Macros
----------------------------------------------------------
ENABLE KEYBOARD: Press and hold key 5 
and sw1 to enable the keyboard. 
Release once the cadence LEDs become active.

All remaining keystrokes require 1) depressing 
the indicated keys while the green LED in on, and
2) holding those keys down until the red LED flashes.

SPEED: sw1

BEACON: key 4 + sw1
        Edit the text under '//Beacon'.

XMIT ON: key 1 + sw1

XMIT OFF: key 2 + sw1

CHANGE BAND: key 4 + key 5 + sw1

QSY UP 1000 Hz: key 1 + key 3 + key 4 + key 5 + sw1

QSY DOWN 1000 HZ: key 1 + key 2 + key 3 + key 4 + key 5 + sw1

QSY UP 100 Hz: key 1 + key 3 + sw1

QSY DOWN 100 HZ: key 1 + key 2 + key 3 + sw1

QSY UP 10 Hz: key 1 + key 3 + key 4 + sw1

QSY DOWN 10 HZ: key 1 + key 2 + key 3 + key 4 + sw1

ENTER FQY FROM PC OR BAUDOT KEYBOARD: key 1 + key 4 + sw1

TEANSMIT TEXT FROM PC OR BAUDOT KEYBOARD: key 2 + key 4 + sw1

CONTEST MACRO 1: key 1 + key 2 + sw1
        Edit the text under '//Macro 1'.
        
CONTEST MACRO 2: key 2 + key 3 + sw1
        Increments contact number
        Edit the text under '//Macro 2'.

REPEAT MACRO 2: key 3 + sw1
        Does not increment contact number
        Edit the text under '//Macro 2'.

RESET NUM (Contest QSO serial number) TO 0 
in EEPROM: key 1 + key 2 + key 4 + sw1
------------------------------------------------------------
 
   Upload this sketch to an Arduino Leonardo, then CLOSE or MINIMIZE the sketch, to avoid typing into the code!
   On your PC, open an application like Word, Notepad or MMTTY. Press and hold key 5 and sw1 until the cadence 
   LEDs begin flashing. This activates the Baudot Keyboard. Cycle through keyboard speeds using sw1.
    
   When the green cadence LED is lit, press the keys for a character and hold them down until the red LED blinks once.
   (The green LED correspond to the 'cadence tap' heard by an operator using a Baudot Keyboard on a multiplexed teletype system, 
   letting the operator know it's time to send; the red LED corresponds to the distributor brush reading the state of the operator's 
   keys, sending a positive voltage pulse for unpressed keys, and a negative pulse for pressed keys.)

   All functions of the rtty transmitter are controlled using the keyboard. Refer to Keyboard Macros above. 

   See 'Baudot Multiplex Type-printing System', London: H.M. Stationery Office, 1919,available at 
   http://www.samhallas.co.uk/repository/telegraph/b6_baudot_multiplex.pdf . 
   
   Also see 'Telegraphy' by J. W. Freebody, London: Pitman, 1961, from your public library through interlibrary loan.

*/

// --------------------------------------------Things for you to change--------------------------
//Contest exchange defines
#define myCall "N4RLI"
#define myName "RALPH"
#define RST "599"
#define mySPC "VA"
#define myGridSquare "FM08KD"

//beacon defines
#define beaconLineOne "N4RLI N4RLI N4RLI MEPT TRANSMISSION"
#define beaconLineTwo "5W INTO LOW INVERTED VEE IN STAUNTON VA FM08KD"

//LSB Receive freq - Transmit freq in Hz for each band; include '-' for negative values
//Check this somewhere in the middle of the bandplan allotment

#define corr40 2360           //Hz My Adafruit Si5351 frequency correction on 40m -- yours will be different!
#define corr20 2420           //Hz My Adafruit Si5351 frequency correction on 20m -- yours will be different!
//#define corr40 2880         //Hz My Etherkit Si5351 frequency correction on 40m -- yours will be different!
//#define corr20 3450         //Hz My Etherkit Si5351 frequency correction on 20m -- yours will be different!

//------------------------------------------------------------------------------------------------

//All baudot encoding and string, byte, and bit transmission code due to George Smart M1GEO
#define ARRAY_LEN 32

//Baudot control characters
#define LETTERS_SHIFT 31
#define FIGURES_SHIFT 27
#define LINEFEED 2
#define CARRRTN  8
#define SPACE  4

//Digital pins for keyboard and speed button 
#define key1 5 //7 for pushbutton setup
#define key2 6
#define key3 7 //5 for puchbutton setup
#define key4 11
#define key5 12
#define sw1 4
#define Offset 170          //Hz Added to frequency above to give 'mark' freq
                           
//#define bitDelay 20          //delay for transmitting bits,in milliseconds, to achieve 
#define bitDelay 18            //45.45 baud data transmission rate -- found experimentally (22 ms does NOT work)

//bandwise bounds for rtty according to ARRL band plan as of February 2019
#define fMax40 7125000
#define fMin40 7080000
#define fMax20 14099500
#define fMin20 14070000

// Digital pins for cadence LEDs
#define greenLED 10
#define redLED 9

#define is_lowercase(ch)    ((ch) >= 'a' && (ch) <= 'z')
#define is_uppercase(ch)    ((ch) >= 'A' && (ch) <= 'Z')

#include <EEPROM.h>
#include "Wire.h"
#include "si5351_112.h"
#include <stdint.h>
#include <Keyboard.h>

int eeNUMAddress = 0; //address of stored NUM in EEPROM
int eeBandAddress = sizeof(int); //address of stored Band in EEPROM
int eecurrFAddress = 2*sizeof(int); //address of stored currF in EEPROM

//Default band for transmission
int Band = 40;

//Default rtty bounds
unsigned long fMax = fMax40;
unsigned long fMin = fMin40;

int corr;

char inData[101];
char inChar;
int index = 0;

// initialize states of keys and speed button
int key1State = 1;
int key2State = 1;
int key3State = 1;
int key4State = 1;
int key5State = 1;
int sw1State = 1;

int FIG = 0;
int LTR = 1;  //by default, the Baudot keyboard is in letter mode 

//Default speed = 5 WPM. 
int keyDelay = 2105; //Time for operator to press keys, in milliseconds
int xmitDelay = 100; //Delay between character transmissions, in milliseconds
int blinkDelay = 20; //Delay to give red LED a visible blink,, in milliseconds

//Press sw1 to change speed.
int sendSpeed = 5;

// while startMainLoop is 0, the keyboard is disabled
int startMainLoop = 0;
//RTTY transmit initially off
int XMIT = 0;

int NUM = 0; //contest contact number (max 100)
char NUMTXT[10]; //char version of NUM

char letters_arr[33] = "\000E\nA SIU\rDRJNFCKTZLWHYPQOBG\000MXV\000";
char figures_arr[33] = "\0003\n- \a87\r$4',!:(5\")2#6019?&\000./;\000";

int ita1Value = 0;
int letters_ita1_to_ita2[32] = {0,3,1,4,21,7,6,24,27,11,26,20,25,14,13,9,31,0,29,17,5,16,19,30,0,15,28,18,10,23,12,22};
int figures_ita1_to_ita2[32] = {0,23,19,4,1,10,0,16,27,21,7,0,6,24,0,22,31,28,12,14,30,13,25,5,0,15,18,0,3,29,0,0};
int letters_ita1_to_ascii[32] = {0,65,69,32,89,85,73,79,0,74,71,72,66,67,70,68,0,8,88,90,83,84,87,86,0,75,77,76,82,81,78,80};
int figures_ita1_to_ascii[32] = {0,49,50,32,51,52,0,53,0,54,55,0,56,57,0,48,0,46,44,58,59,33,63,39,0,40,41,0,45,47,0,0};

Si5351 si5351;

unsigned long currF; //current frequency -- needs corr (correction term) subtracted to correspond with lsb receiver frequency

void setup()
{
  //Initialize Si5351  
  //si5351.reset();
  si5351.init(SI5351_CRYSTAL_LOAD_8PF, 0);
  si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);
  si5351.drive_strength(SI5351_CLK0, SI5351_DRIVE_8MA);
  si5351.set_clock_pwr(SI5351_CLK0, 0);
  si5351.output_enable(SI5351_CLK0, 1); 
  
  //Get last contest contact serial number, last band used, last frequency used from eeprom
  EEPROM.get(eeNUMAddress,NUM);
  EEPROM.get(eeBandAddress,Band);
  getBounds();
  EEPROM.get(eecurrFAddress,currF);
  
// configure arduino data pins for keyboard input
  pinMode(key1,INPUT_PULLUP);
  pinMode(key2,INPUT_PULLUP);
  pinMode(key3,INPUT_PULLUP);
  pinMode(key4,INPUT_PULLUP);
  pinMode(key5,INPUT_PULLUP);

// configure pin for speed button sw1
  pinMode(sw1,INPUT_PULLUP); 

// configure data pins for cadence LEDs
  pinMode(redLED,OUTPUT);
  pinMode(greenLED,OUTPUT);
  digitalWrite(greenLED,HIGH);
  Keyboard.begin();
  
}

void loop()
{  
         
 //Prevent automatic startup of main loop
  while(!startMainLoop){
    //do nothing until key5 and sw1 are depressed
    key5State = (digitalRead(key5)?0:1);
    sw1State = (digitalRead(sw1)?0:1);      

    if(key5State && sw1State){ //When user presses key5 and sw1, turn on keyboard
                               //and print current transmit frequency
      Keyboard.print(currF + corr); 
      startMainLoop = 1;
    }
  } 
  
  diddleDelay(xmitDelay);


  digitalWrite(greenLED, HIGH);  //alert operator to depress keys
  diddleDelay(keyDelay); //transmit diddles while green LED is on

  digitalWrite(redLED,HIGH); //alert operator that key states are being read
  key1State = (digitalRead(key1)?0:1); //read key states .. reverse values due to pull up
  key2State = (digitalRead(key2)?0:1);
  key3State = (digitalRead(key3)?0:1);
  key4State = (digitalRead(key4)?0:1);
  key5State = (digitalRead(key5)?0:1);
  sw1State = (digitalRead(sw1)?0:1);

  diddleDelay(blinkDelay); //transmit diddles while red LED blinks

  digitalWrite(redLED,LOW);  //turn off LEDs
  digitalWrite(greenLED,LOW);

//***Translate key states into ita1; convert ita1 to ita2; transmit ita2;convert ita1 to ascii; send ascii to PC word processing ap ***
// *** BEGIN MAIN CONDITIONAL ***
 if(!sw1State){

   //Compute integer value of 5 bit key entry; key 1 is least significant bit
   ita1Value = 1*key1State + 2*key2State + 4*key3State + 8*key4State + 16*key5State;

    if(ita1Value == 16){
      LTR = 1;
      FIG = 0;
    }
    
    if(ita1Value == 8){
      LTR = 0;
      FIG = 1;
    }

if(ita1Value == 24){ //Modification of Baudot Code -- replacing error symbol with carriage return and linefeed

 rtty_txbyte(8);
 rtty_txbyte(2);
 Keyboard.write(13);
 Keyboard.write(10);
  
}
    
    if(LTR){
      rtty_txbyte(letters_ita1_to_ita2[ita1Value]);
      Keyboard.write(letters_ita1_to_ascii[ita1Value]);
    }
    
    if(FIG){
      rtty_txbyte(figures_ita1_to_ita2[ita1Value]);  
      Keyboard.write(figures_ita1_to_ascii[ita1Value]);    
    }
 
 }else{

//************************GENERAL MACROS***********************************************************************************

 // Turn on RTTY transmit
 if(key1State && !key2State && !key3State && !key4State && !key5State && sw1State){

    beginTransmit();
 }

 // Turn off RTTY transmit
 if(!key1State && key2State && !key3State && !key4State && !key5State && sw1State){

    endTransmit();
 }

// Keyboard speed:
  // 1 char/(keyDelay + xmitDelay + blinkDelay + 1720) ms  *  1000 ms/ sec  *  60 sec/ min  *  1 word/5 char =
  // 12000/(keyDelay + xmitDelay + blinkDelay + 1720) = WPM   
  // (1720 ms is approximately the time required to process one char. It was found experimentally. It is an average
  // of one minute sending 'A' without transmitting diddles, and one minute of sending 'A' while transmitting diddles.
  // In both cases, keyDelay and xmitDelay and blinkDelay were set to 0.)

//Cycle through speeds
if (!key1State && !key2State && !key3State && !key4State && !key5State && sw1State){

  switch (sendSpeed) {
    case 20:
     sendSpeed = 5; 
     keyDelay = 2105;
     xmitDelay = 100;
     blinkDelay = 20;
     for(int i=1;i<=8;i++){
      Keyboard.write(KEY_BACKSPACE);
     }
     Keyboard.print("   5 WPM");
    break;
    case 15:
     sendSpeed = 20; 
     keyDelay = 305;
     xmitDelay = 100;
     blinkDelay = 20;
     for(int i=1;i<=8;i++){
      Keyboard.write(KEY_BACKSPACE);
     }
     Keyboard.print("  20 WPM");
    break;
    case 10:
     sendSpeed = 15; 
     keyDelay = 505;
     xmitDelay = 100;     
     blinkDelay = 20;
     for(int i=1;i<=8;i++){
      Keyboard.write(KEY_BACKSPACE);
     }
     Keyboard.print("  15 WPM");
    break;
    case 5:
     sendSpeed = 10; 
     keyDelay = 905;
     xmitDelay = 100;
     blinkDelay = 20;
     for(int i=1;i<=8;i++){
      Keyboard.write(KEY_BACKSPACE);
     }
     Keyboard.print("  10 WPM");
    break;
    default: 
     sendSpeed = 5;  
     keyDelay = 2105;
     xmitDelay = 100;
     blinkDelay = 20;
     for(int i=1;i<=8;i++){
      Keyboard.write(KEY_BACKSPACE);
     }
     Keyboard.print("   5 WPM");
  }
}

 //Beacon
 if (!key1State && !key2State && !key3State && key4State && !key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();
    
     for(int i=1;i<5;i++){
        rtty_txbyte(LINEFEED);
        rtty_txbyte(CARRRTN);
        rtty_txbyte(SPACE);
        rtty_txstring(beaconLineOne);
        rtty_txbyte(SPACE);
        rtty_txbyte(LINEFEED);
        rtty_txbyte(CARRRTN);
        rtty_txbyte(SPACE);
        rtty_txstring(beaconLineTwo);
        rtty_txbyte(SPACE);
        rtty_txbyte(LINEFEED);
        rtty_txbyte(CARRRTN);
     }

    endTransmit();
    digitalWrite(redLED,LOW);
}

//************************************************VFO MACROS****************************************************** 

 // QSY UP 10 Hz
 if(key1State && !key2State && key3State && !key4State && !key5State && sw1State && !XMIT){
    if(currF + corr + 10UL <= fMax){
      currF += 10UL;
    } 
      int i;
      for(i=1;i<=8;i++){
         Keyboard.write(KEY_BACKSPACE);       
      }
        if(Band >= 40){Keyboard.print(" ");}
         Keyboard.print(currF + corr);
 }

  // QSY DOWN 10 Hz
 if(key1State && key2State && key3State && !key4State && !key5State && sw1State && !XMIT){
  
    if(currF + corr - 10UL >= fMin){
      currF -= 10UL;
    } 
      int i;
      for(i=1;i<=8;i++){
         Keyboard.write(KEY_BACKSPACE);       
      }  
        if(Band >= 40){Keyboard.print(" ");}
         Keyboard.print(currF + corr);
 }

  // QSY UP 100 Hz
 if(key1State && !key2State && key3State && key4State && !key5State && sw1State && !XMIT){
    if(currF + corr + 100UL <= fMax){
      currF += 100UL;
    } 
      int i;
      for(i=1;i<=8;i++){
         Keyboard.write(KEY_BACKSPACE);       
      }       
        if(Band >= 40){Keyboard.print(" ");}
         Keyboard.print(currF + corr);
 }

  // QSY DOWN 100 Hz
 if(key1State && key2State && key3State && key4State && !key5State && sw1State && !XMIT){
  
    if(currF + corr - 100UL >= fMin){
      currF -= 100UL;
    } 
      int i;
      for(i=1;i<=8;i++){
         Keyboard.write(KEY_BACKSPACE);       
      }  
        if(Band >= 40){Keyboard.print(" ");}
         Keyboard.print(currF + corr);
 }

  // QSY UP 1000 Hz
 if(key1State && !key2State && key3State && key4State && key5State && sw1State && !XMIT){
    if(currF + corr + 1000UL <= fMax){
      currF += 1000UL;
    } 
      int i;
      for(i=1;i<=8;i++){
         Keyboard.write(KEY_BACKSPACE);       
      }
        if(Band >= 40){Keyboard.print(" ");}
         Keyboard.print(currF + corr);
 }

  // QSY DOWN 1000 Hz
 if(key1State && key2State && key3State && key4State && key5State && sw1State && !XMIT){
  
    if(currF + corr - 1000UL >= fMin){
      currF -= 1000UL;
    } 
      int i;
      for(i=1;i<=8;i++){
         Keyboard.write(KEY_BACKSPACE);       
      }  
        if(Band >= 40){Keyboard.print(" ");}
         Keyboard.print(currF + corr);
 }


// Change Band
 if(!key1State && !key2State && !key3State && key4State && key5State && sw1State && !XMIT){
    switch(Band){
      case 40:
          Band = 20;
          getBounds(); 
          currF = 14090000 - corr;          
      break;
      case 20:
          Band = 40;
          getBounds();
          currF = 7090000 - corr;
      break;
      default:
          Band = 40;
          getBounds();                
          currF = 7090000 - corr;
    }
      int i;
      for(i=1;i<=8;i++){
         Keyboard.write(KEY_BACKSPACE);       
      }           
        if(Band >= 40){Keyboard.print(" ");}
         Keyboard.print(currF + corr);
         
      EEPROM.put(eeBandAddress,Band);
      EEPROM.put(eecurrFAddress,currF);
 }

 //Enter frequency from PC Keyboard 
 if(key1State && !key2State && !key3State && key4State && !key5State && sw1State && !XMIT){
  enterFqyByPC();
 }

//*********************************CONTEST MACROS***************************************************

//Macro 1
if (key1State && key2State && !key3State && !key4State && !key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();
      
    rtty_txstring(" \r\n ");
    rtty_txstring(myCall);\
    rtty_txbyte(SPACE);
    rtty_txstring(myCall);
    rtty_txstring(" \r\n ");
    
    endTransmit();
    digitalWrite(redLED,LOW);
}

//Macro 2
if (!key1State && key2State && key3State && !key4State && !key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();
    NUM++; //increment serial contact number
    
    sendContestExchange();         

    //write serial number to eeprom
    EEPROM.put(eeNUMAddress,NUM);

    endTransmit();
    digitalWrite(redLED,LOW);
}

 //Repeat Macro 2, without incrementing serial contact number
   if (!key1State && !key2State && key3State && !key4State && !key5State && sw1State){

      digitalWrite(redLED,HIGH);
      beginTransmit();
      sendContestExchange();          
      endTransmit();
      digitalWrite(redLED,LOW);
}

 //Reset NUM to 0 in EEPROM
 if (key1State && key2State && !key3State && key4State && !key5State && sw1State){
     NUM = 0;
    EEPROM.put(eeNUMAddress,NUM);
    sprintf(NUMTXT,"%03d",NUM); //Gives leading zeros for values less than 100, as required by some contest rules
    for(int i=1;i<=8;i++){
      Keyboard.write(KEY_BACKSPACE);
    }
    Keyboard.print(NUMTXT);
    
 }

//***********************************QSO MACROS***************************************************

//CQ 2x2
if (key1State && !key2State && !key3State && key4State && key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();
      
    rtty_txstring(" \r\n ");
    rtty_txstring(" CQ CQ DE  ");
    rtty_txstring(myCall);
    rtty_txstring(myCall);
    rtty_txstring(" \r\n ");
    
    endTransmit();
    digitalWrite(redLED,LOW);
}

//CQ 3X3
if (key1State && key2State && !key3State && key4State && key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();
      
    rtty_txstring(" \r\n ");
    rtty_txstring(" CQ CQ CQ DE  ");
    rtty_txstring(myCall);
    rtty_txstring(myCall);
        rtty_txstring(myCall);
    rtty_txstring(" \r\n ");
    
    endTransmit();
    digitalWrite(redLED,LOW);
}

//QRZ?
if (!key1State && !key2State && !key3State && !key4State && key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();
      
    rtty_txstring(" \r\n ");
    rtty_txstring(" QRZ? DE ");
    rtty_txstring(myCall);
    rtty_txstring(" \r\n ");
    
    endTransmit();
    digitalWrite(redLED,LOW);
}

//QSO
if (!key1State && !key2State && key3State && !key4State && key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();

    rtty_txstring(" \r\n ");
    sendTxtByPC(); //send CALLSIGN    
    rtty_txbyte(SPACE);    
    sendTxtByPC(); //send RST
    rtty_txbyte(SPACE);
    sendTxtByPC(); //send RST
    rtty_txbyte(SPACE);
    rtty_txstring(" IN ");
    rtty_txstring(mySPC);    
    rtty_txbyte(SPACE);
    rtty_txstring(mySPC);  
    rtty_txstring(" OP ");
    rtty_txbyte(SPACE);
    rtty_txstring(myName); 
    rtty_txbyte(SPACE);
    rtty_txstring(myName); 
    rtty_txbyte(SPACE);
    rtty_txstring(" HW? ");
    rtty_txstring(" DE "); 
    rtty_txstring(myCall);
    rtty_txstring(" \r\n ");
    
    endTransmit();
    digitalWrite(redLED,LOW);
}

//AGN
if (!key1State && !key2State && key3State && !key4State && key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();
      
    rtty_txstring(" \r\n ");
    rtty_txstring(" AGN? DE ");
    rtty_txstring(myCall);
    rtty_txstring(" \r\n ");
    
    endTransmit();
    digitalWrite(redLED,LOW);
}

//73
if (!key1State && key2State && key3State && key4State && !key5State && sw1State){

    digitalWrite(redLED,HIGH);
    beginTransmit();

    rtty_txstring(" \r\n ");
    rtty_txstring(" QSL ON ALL ");    
    sendTxtByPC(); //send NAME       
    rtty_txstring(". THANKS FOR THE QSO. NOW 73 AND GOOD LUCK TO YOU.");
    rtty_txstring(" DE "); 
    rtty_txstring(myCall);
    rtty_txstring(" 73 \r\n ");
    
    endTransmit();
    digitalWrite(redLED,LOW);
}

//TRANSMIT BUFFERED TEXT

 if(!key1State && key2State && !key3State && key4State && !key5State && sw1State && !XMIT){
     digitalWrite(redLED,HIGH);
    beginTransmit();

    rtty_txstring(" \r\n ");
    rtty_txbyte(SPACE);
    
    sendTxtByPC(); //send text

    rtty_txstring(" \r\n ");
    rtty_txbyte(SPACE);

    endTransmit();
    digitalWrite(redLED,LOW);
 }


//*******************************************EXTRA MEMORIES*************************************************

//MEM1
if(key1State && !key2State && !key3State && !key4State && key5State && sw1State){
   //Your code here
 }

//MEM2
if(!key1State && key2State && !key3State && !key4State && key5State && sw1State){
   //Your code here
 }
 
//MEM3
if(key1State && !key2State && key3State && !key4State && key5State && sw1State){
   //Your code here
 }
 
//MEM4
if(key1State && key2State && key3State && !key4State && key5State && sw1State){
   //Your code here
 }
 
//MEM5
if(!key1State && key2State && key3State && !key4State && key5State && sw1State){
   //Your code here
 }
 
//MEM6
if(key1State && key2State && !key3State && !key4State && key5State && sw1State){
   //Your code here
 }
 
//MEM7
if(!key1State && key2State && key3State && key4State && key5State && sw1State){
   //Your code here
 }
 
//MEM8
if(!key1State && !key2State && key3State && key4State && key5State && sw1State){
   //Your code here
 }
 
//MEM9
if(!key1State && key2State && !key3State && key4State && key5State && sw1State){
   //Your code here
 }
 
 } // *** END MAIN CONDITIONAL ***

} //end loop

//------------------------data structure and function definitions----------------------------------------------

enum baudot_mode
{
  NONE,
  LETTERS,
  FIGURES
};

void diddleDelay(int delayTime){ //transmit nonprinting characters, called diddles, during delays
  int delaySoFar = 0;

  while(delayTime - delaySoFar > 0){
    if(delayTime - delaySoFar >= 172){      
      if(LTR){
        rtty_txbyte(LETTERS_SHIFT);
      }else if(FIG){
        rtty_txbyte(FIGURES_SHIFT); 
      }
        delaySoFar += 172 ;  //ms measured system time usage per byte
    }else{
        delay(delayTime - delaySoFar);
        delaySoFar = delayTime;
    }
  }  
}

void getBounds(){
  switch(Band){
    case 40:
      fMax = fMax40; //with 
      fMin = fMin40;
      corr = corr40;
    break;
    case 20:
      fMax = fMax20;
      fMin = fMin20;
      corr = corr20;
    break;
    default:
      fMax = fMax40;
      fMin = fMin40;
      corr = corr40;
  }
}

void enterFqyByPC(){
  //waiting for input
Keyboard.end();
Serial.begin(9600);
  while(!Serial){
    //wait
  }  
Serial.println("Type frequency in Hz, then press Enter");
while (Serial.available() == 0){
  //Wait for user input
}
unsigned long val = Serial.parseInt(); //read int entered by PC keyboard
if(val >= fMin && val <= fMax){ //check that entered fqy is within bandplan bounds
      currF = val - corr;
      Serial.println(currF + corr);
    }else{
      Serial.println("Out of bandplan bounds");
    }
//flush read buffer
  while(Serial.available() > 0){
    Serial.read();
  }
  Serial.end();
  Keyboard.begin();
}

void sendTxtByPC(){
  Keyboard.end();
  Serial.begin(9600);
  while(!Serial){
    //wait for Serial to start
  }
while(Serial.read() >= 0) ; // flush the receive buffer

  Serial.println("Type text, then press Enter");

  while(Serial.available() == 0){ 
    //Send diddles while waiting for user to enter text
        if(LTR){
          rtty_txbyte(LETTERS_SHIFT);
        }else if(FIG){
          rtty_txbyte(FIGURES_SHIFT); 
        } 
  }

while(Serial.available() > 0) // Read characters while there is data                                                
  {
      if(index < 101) // One less than the size of the array
      {
          inChar = Serial.read(); // Read a character
          inData[index] = inChar; // Store it
          index++; // Increment where to write next
          inData[index] = '\0'; // Null terminate the string
      }
  }

  if(inData[0] > 0) {
    rtty_txstring(inData);
    }
while(Serial.read() >= 0) ; // flush the receive buffer
  Serial.end();
  Keyboard.begin();
  Keyboard.print(inData); //echo text to serial monitor input buffer
  
  //reinitialize char array
  inData[0] = 0; 
  index = 0;
}

void beginTransmit(){
      XMIT = 1;
      si5351.set_clock_pwr(SI5351_CLK0, 1); //turn on power to clk0
      delay(100);
      si5351.set_freq(currF + Offset, 0ULL,SI5351_CLK0);
      delay(100);  
}

void endTransmit(){
      XMIT = 0;
      si5351.set_clock_pwr(SI5351_CLK0, 0); //turn off power to clk0
      EEPROM.put(eecurrFAddress,currF);
}

void sendContestExchange(){

// For CQ WW RTTY
    //rtty_txstring(" \r\n ");
    //rtty_txbyte(RST);
    //rtty_txbyte(SPACE);
    //sprintf(NUMTXT,"%03d",NUM); //Gives leading zeros for values less than 100, as required by some contest rules 
    //rtty_txstring(NUMTXT);
    //rtty_txbyte(SPACE);
    //rtty_txstring(NUMTXT);
    //rtty_txbyte(SPACE);
    //rtty_txstring("DE");
    //rtty_txbyte(SPACE);
    //rtty_txstring(myCall); 
    //rtty_txstring(" \r\n ");

//For NCJ NA QSO PARTY RTTY
    //rtty_txstring(" \r\n ");
    //rtty_txstring(myName); 
    //rtty_txbyte(SPACE);
    //rtty_txstring(mySPC); 
    //rtty_txbyte(SPACE);
    //rtty_txstring(myName);
    //rtty_txbyte(SPACE); 
    //rtty_txstring(mySPC);
    //rtty_txbyte(SPACE);
    //rtty_txstring("DE");
    //rtty_txbyte(SPACE);
    //rtty_txstring(myCall);
    //rtty_txstring(" \r\n ");

//For BARTG HF
    rtty_txstring(" \r\n ");
    rtty_txstring(RST);
    rtty_txbyte(SPACE);
    sprintf(NUMTXT,"%03d",NUM); //Gives leading zeros for values less than 100, as required by some contest rules 
    rtty_txstring(NUMTXT);
    rtty_txbyte(SPACE);
    rtty_txstring(NUMTXT); 
    rtty_txbyte(SPACE);
    sendTxtByPC(); //User sends 4-digit UTC time
    rtty_txbyte(SPACE);
    sendTxtByPC(); //User sends 4-digit UTC time
    rtty_txbyte(SPACE);
    rtty_txstring("DE");
    rtty_txbyte(SPACE);
    rtty_txstring(myCall);
    rtty_txstring(" \r\n ");

//For VA QSO PARTY
    //rtty_txstring(" \r\n ");
    //sprintf(NUMTXT,"%03d",NUM); //Gives leading zeros for values less than 100, as required by some rtty contests 
    //rtty_txstring(NUMTXT);
    //rtty_txbyte(SPACE);
    //rtty_txstring(NUMTXT);
    //rtty_txbyte(SPACE);
    //rtty_txstring("STX"); 
    //rtty_txbyte(SPACE);
    //rtty_txstring("STX"); 
    //rtty_txbyte(SPACE);
    //rtty_txstring(myCall);
    //rtty_txstring(" \r\n ");  
}
 
void rtty_txstring(char *str)
{
  enum baudot_mode current_mode = NONE;
  char c;
  uint8_t b;
 
  while (*str != '\0')
  {
    c = *str;
    
    /* some characters are available in both sets */
    if (c == '\n')
    {
      rtty_txbyte(LINEFEED);
    }
    else if (c == ' ')  //Irons 2/27/2019
    {
      rtty_txbyte(SPACE);      
    }
    else if (c == '\r')
    {
      rtty_txbyte(CARRRTN);
    }
    else if (is_lowercase(*str) || is_uppercase(*str))
    {
      if (is_lowercase(*str))
      {
        c -= 32;
      }
 
      if (current_mode != LETTERS)
      {
        rtty_txbyte(LETTERS_SHIFT);
        current_mode = LETTERS;
      }
 
      rtty_txbyte(char_to_baudot(c, letters_arr));
    }
    else
    {
      b = char_to_baudot(c, figures_arr);
 
      if (b != 0 && current_mode != FIGURES)
      {
        rtty_txbyte(FIGURES_SHIFT);
        current_mode = FIGURES;
      }
 
      rtty_txbyte(b);
    }
 
    str++;
  }
}

uint8_t char_to_baudot(char c, char *array)
{
  int i;
  for (i = 0; i < ARRAY_LEN; i++)
  {
    if (array[i] == c)
      return i;
  }
 
  return 0;
}
 
void rtty_txbyte(uint8_t b)
{ 
  int8_t i;
 
  rtty_txbit(0); //1 start bit
 
  for (i = 0; i < 5; i++) // 5 data bits
  {
    if (b & (1 << i))
      rtty_txbit(1);
    else
      rtty_txbit(0);
  }
 
  rtty_txbit(1); // 2 stop bits
  rtty_txbit(1); 
} 

// Transmit a bit as a mark or space
void rtty_txbit (int bit) {
  if (bit) {
    // High - mark
    //digitalWrite(2, HIGH);
    //digitalWrite(3, LOW);
    
    sendFrequency_mark(currF);
    
  } else {
    // Low - space
    //digitalWrite(3, HIGH);
    //digitalWrite(2, LOW);
    
    sendFrequency_space(currF);
    
  }
  
  // Delay appropriately - tuned to 45.45 baud.
  delay(bitDelay); //sets the baud rate
}

void sendFrequency_mark(unsigned long f)
{ 
   unsigned long fqy = (f + Offset)*100UL;
         
          if(XMIT){si5351.set_freq(fqy, 0ULL, SI5351_CLK0);}         
}

void sendFrequency_space(unsigned long f)
{ 
   unsigned long fqy = (f)*100UL;
   
          if(XMIT){si5351.set_freq(fqy, 0ULL ,SI5351_CLK0);}  

}
